% Generate reduced basis using a greedy algorithm based on maximum
% residual of the (current) reduced basis solution, where the search
% is done over a set of n_search randomly chosen parameters (the training
% set

% Size of spatial discretization, number of parameters
dim_sp  = nint;   %length(K{1});
dim_par = noarv;
fxi = fnew(1:dim_sp);   

% Training set of parameters
n_train_r = 3000;
tol_rb = (1.d-5)*norm(fxi(1:dim_sp));

fprintf('Building reduced basis using %i randomly generated training points\n',n_train_r); 
xi_set = 2*rand(dim_par,n_train_r) - 1;

% Maximum size of pre-allocated reduced basis
max_rdim = 100;
Q = zeros(dim_sp,max_rdim);

% Initialize reduced basis using mean of parameter set (=0)
Kxi = K{1};
u = Kxi\fxi;
Q(:,1) = u/norm(u);
rb_size = 1;
Qrb_rand = Q(:,1:rb_size);

% Build reduced basis on the fly
for k=1:n_train_r,
% For kth member of training set
   xi = xi_set(:,k);
   Kr_rand = K_reduced(K,Qrb_rand,dim_par);
   Kxi   = K{1};
   Kr_xi = Kr_rand{1};
   for j=1:dim_par,
      Kxi   = Kxi   + K{j+1}*xi(j);
      Kr_xi = Kr_xi + Kr_rand{j+1}*xi(j);
   end
% Compute the reduced solution and the norm of its residual
   u_rb = Qrb_rand*(Kr_xi\(Qrb_rand'*fxi(1:dim_sp)));
   r = norm(fxi-Kxi*u_rb);
% If error indicator is too large, compute full solution and use it to
% augment the reduced basis
   if r>tol_rb,
      u = Kxi\fxi(1:dim_sp);
      rb_size = rb_size+1;
      Q(:,rb_size) = u;
      Qrb_rand = orth(Q(:,1:rb_size));
   end
end
   
fprintf('Reduced basis of size %i constructed \n',rb_size);